#include "Processors.h"

namespace armarx::armem::server::ltm
{
    Processors::Processors()
    {
        // setup containers
        memFilters.push_back(&memFreqFilter);
        snapFilters.push_back(&snapFreqFilter);
        snapFilters.push_back(&snapEqFilter);
        extractors.push_back(&imageExtractor);
        extractors.push_back(&depthImageExtractor);
        converters.insert({pngConverter.identifier, &pngConverter});
        converters.insert({exrConverter.identifier, &exrConverter});
    }

    void Processors::createPropertyDefinitions(PropertyDefinitionsPtr& defs, const std::string& prefix)
    {
        // filters
        defs->optional(memFreqFilter.enabled, prefix + "memFreqFilter.Enabled");
        defs->optional(memFreqFilter.waitingTimeInMs, prefix + "memFreqFilter.WaitingTime", "Waiting time in MS after each LTM update.");
        defs->optional(snapFreqFilter.enabled, prefix + "snapFreqFilter.Enabled");
        defs->optional(snapFreqFilter.waitingTimeInMs, prefix + "snapFreqFilter.WaitingTime", "Waiting time in MS after each Entity update.");
        defs->optional(snapEqFilter.enabled, prefix + "snapEqFilter.Enabled");
        defs->optional(snapEqFilter.maxWaitingTimeInMs, prefix + "snapEqFilter.MaxWaitingTime", "Max Waiting time in MS after each Entity update.");

        // extractors
        defs->optional(imageExtractor.enabled, prefix + "imageExtractor.Enabled");
        defs->optional(depthImageExtractor.enabled, prefix + "depthImageExtractor.Enabled");

        // converters
        defs->optional(pngConverter.enabled, prefix + "pngConverter.Enabled");
        defs->optional(exrConverter.enabled, prefix + "exrConverter.Enabled");
    }
}
